/*****************************************************************************************
 * $Id: orva_freq_trend_line.sas,v 1.1.2.7.8.1 2012/05/03 13:24:09 sasdje Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    orva_freq_trend_line.sas                                             
 * PURPOSE: 
 *          
 * SUPPORT: sasdje
 *
 * PARAMETERS:
 *
 *     dataM= loss data,
 *
 *     ccDataM= ccuid, ccvuid_0, ccvuid_1, etc.,
 *
 *     NUMCCVS      = ,
 *
 *     outdata= has one row per CCUID (regardless whether there exist losses for
 *              that CCUID).  Columns are:  reason, status, CCUID, intercept, slope,
 *              intercept_Probt, slope_Probt.
 *
 *
 *     trendStatus= 
 *
 *   
 *****************************************************************************************/



%macro orva_freq_trend_line( 
                                       dataM        = ,
                                       ccDataM      = ,
									   NUMCCVS      = ,
                                       interval     = ,
									   global_range = ,
                                       outdata      = ,
                                       trendStatus  = 
                                       );

	/*** create the status data set upfront **/
	%let reason = %sysfunc(sasmsg(&orvads, unknown_error, noquote));    
	%orva_status_ds( statusds = &trendStatus, 
                     status   = 1, 
                     reason   = &reason 
	); 
	
   	/*- error checking -*/
    %if &global_range=%str() %then %do;
        %put &orvar_error %sysfunc(sasmsg(&orvads, gfmom_specify_global_range_ds_error, noquote));
        %let abort=1;
        %let reason=%sysfunc(sasmsg(&orvads, gfmom_specify_global_range_ds_error, noquote));
        %goto DONE;
    %end;
    data _null_; set &global_range; 
        call symput( "global_start", start_date);
    run;

	%let status = 0;
	%let reason = %sysfunc(sasmsg(&orvads, successful_note, noquote));
	%let reasonOneCell = %sysfunc(sasmsg(&orvads, successful_note, noquote));

	data &outdata;
		set _NULL_;
	run;


	%let nobs = 0;
	%orvu_getObsCount(&dataM, nobs);
	%if &nobs EQ 0 %then %do;
		%put &orvar_error %sysfunc(sasmsg(&orvads, ftl_loss_data_empty_error, noquote));
		%let status  = 1;
		%let reason = %sysfunc(sasmsg(&orvads, ftl_loss_data_empty_error, noquote));
		%goto DONE;
	%end;

        %let sevmoments=varprj.sevmomts_i;

	%let nobs_moments = 0;
        data _sevmoments_; set &sevmoments;
        where count > 3;
        keep ccuid;
        run;
	%orvu_getObsCount(_sevmoments_, nobs_moments);
	%if &nobs_moments EQ 0 %then %do;
		%put &orvar_error %sysfunc(sasmsg(&orvads, ftl_cc_data_empty_error, noquote));
		%let status  = 1;
		%let reason = %sysfunc(sasmsg(&orvads, ftl_cc_data_empty_error, noquote));
		%goto DONE;
	%end;
     

        ods listing close;

        %let firstmsg =  %sysfunc(sasmsg(&orvads, successful_note, noquote));

        /**** create subsetting where clauses for all ccuids *****/
        data varprj.cellwhere(index=(ccuid)); 
           set &ccData;
           length where $200;
	   array cc[*] ccvuid_:;

           first = 1;
           do i = 1 to dim(cc);
              if cc[i] ne 0 then do;
                   if first ne 1 then do;
                      where = trim(left(where)) || " and ";
                   end;
			
                   where = trim(left(where)) || " " || vname(cc[i]) || " eq " || trim(left(cc[i]));
			first = 0;
              end;
           end;
      
           if missing(where) then where = "1";
           call symput("NUMCCVS", dim(cc));
           keep ccuid where;
        run;


       	%do index=1 %to &nobs_moments;
		%let statusOneCell = 0;
		%let reasonOneCell = &firstmsg;
		data _ftl_onecell ;
			length reason $ 128;
			reason = "&reasonOneCell";
			status = &statusOneCell;
		run;
                data _NULL_; set _sevmoments_; if _N_ = &index; 
                    call symput("CCUID",ccuid);
                run;
				
		   /**** create subsetting where clause *****/
                %let CELL_WHERE = ;
                data _NULL_; 
                   set varprj.cellwhere;
                   where CCUID = &ccuid;
                   call symput("CELL_WHERE", where);
                run;

		/* subset the losses */
		data _ftl_loss1;
			set &dataM;
			where &CELL_WHERE;
                        keep first_date_of_event amount ccuid;
		run;

		proc sort data=_ftl_loss1 out=_ftl_loss2;
			by FIRST_DATE_OF_EVENT;
		run;
	
		data _ftl_loss3;
			set _ftl_loss2 end=last;
                        retain start_date;
                        if _N_ = 1 then do;
                            start_date = intnx("&interval", FIRST_DATE_OF_EVENT, 0);
                            call symput("start_date", start_of_first_interval);
                        end;
			start_of_curr_fitinterval = intnx("&interval", FIRST_DATE_OF_EVENT, 0);
			end_of_curr_fitinterval = intnx("&interval", FIRST_DATE_OF_EVENT, 1);
			ninterval = intck("&interval", start_date, first_date_of_event);
			x1 = ninterval + (first_date_of_event-start_of_curr_fitinterval+1)/(end_of_curr_fitinterval - start_of_curr_fitinterval);
			y = _N_;
			weight = _N_;
			x2 = x1*x1/2;
			drop start_of_curr_fitinterval end_of_curr_fitinterval;
		run;
	        ods output ParameterEstimates = _ParameterEstimates_;
		proc reg data=_ftl_loss3 outest=_ftl_reg1(keep=x1 x2) ;
			model y = x1 x2 / NOINT;
			weight weight;
		run;
		quit;

		proc transpose data=_ParameterEstimates_(keep=Variable  Probt ) 
			out=_ftl_params2(rename=(x1=intercept_Probt x2=slope_Probt) drop=_NAME_ _LABEL_);
			id Variable; 
		run;

		data _ftl_onecell (rename=(x1=intercept x2=slope));
			length reason $ 128;
			set _ftl_reg1;
		        set _ftl_params2;	
			/* move the Intercept because the location of "x=0" for the histogram is at intnx("&interval", &global_start, 0) */
			x1 = x1 - x2*intck("&interval", intnx("&interval", &global_start, 0), &start_date);
                        reason = "&reasonOneCell";
			status = &statusOneCell;
			CCUID = &CCUID;
		run;

                %if &index = 1 %then 
		%orvu_append(base=&outdata,new=_ftl_onecell,canUseAppend=false,compress=);
                %else 
                %orvu_append(base=&outdata,new=_ftl_onecell,canUseAppend=true,compress=);

	%end;
        ods output close;
        ods listing;
        

%DONE:
	
	%orva_status_ds( statusds = &trendStatus, 
                     status = &status, 
                     reason = &reason, 
                     update = false );
	
%mend orva_freq_trend_line;


/*%orva_freq_trend_line( */
/*                                       dataM        = TMP1.lossi,*/
/*                                       ccDataM      = shortcc,*/
/*									   NUMCCVS      = 2,*/
/*                                       outdata      = snu,*/
/*                                       trendStatus  = snustatus*/
/*                                       );*/
