/*****************************************************************************************
 * Copyright (C), 2008
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_init_globals.sas                                             
 *  PURPOSE: Second half of initial set up for the OpRiskVaR application.
 *           This macro is called by orva_init() shortly after connecting to the server.
 *           
 *  SUPPORT: stchri
 *  
 *  PARAMETERS: 
 *     infolib - we are all the informational datasets?
 *      
 *  NOTES: This macro uses informational distribution datasets created from the
 *         distributions.xml to create numerous global macro variables.
 *         
 *         It also loads (invokes) every distribution's init macro.
 *   
 *****************************************************************************************/
%macro orva_init_globals(infolib = work);
   /***************************************************************/ 
   /***             Setup DATASET_NAME Length                   ***/ 
   /***************************************************************/ 
   %global ORV_DATASET_NAME_LENGTH ; 
   %let    ORV_DATASET_NAME_LENGTH = 41; 

   /***************************************************************/
   /*** setup Severity Distributions Global Macro Vars          ***/
   /***************************************************************/
   %global MAX_SEV_PARAMS ;
   %let    MAX_SEV_PARAMS = 5;  
   
   proc sort data = &infolib..sparmnames;
      by distribution;
   run;
   proc transpose data = &infolib..sparmnames out = _ifs_DistNames;
      var DatasetName;
      by distribution;
   run;
   data _NULL_; 
      set _ifs_DistNames;
      array cols[*] col:;
      call symputX("MAX_SEV_PARAMS", dim(cols));
      stop;
   run;

   %do i=1 %to &MAX_SEV_PARAMS;
      %global param_&i._init;
      %global param_&i.;   
   %end;
   %global Param_pnLimit;
   %global Param_xLimit; 
   
   /*- create global macro variable &dist.._PARAM_LIST_MINUS_1 -*/
   data _NULL_; 
      rc = filename("glob_mac",,"temp"); 
   run;

   data _NULL_;
      length tmpList $128;
      set &infolib..sparmnames; by Distribution;
      file glob_mac;    
      if first.Distribution then do;
         tmpList = compress(Distribution || "_PARAM_LIST_MINUS_1");
         put '%GLOBAL ' tmpList ";";
         Param_List_Minus_1 ="";
      end;
   run;
   %include glob_mac;

   /*- set global macro variable &dist.._PARAM_LIST_MINUS_1 -*/
   data _NULL_;
      set &infolib..sparmnames;
      by Distribution;
      length Param_List_Minus_1 $128;
      retain Param_List_Minus_1 "";
      if first.Distribution then do;
         Param_List_Minus_1 = "";
      end;
      else do;
         Param_List_Minus_1 = strip(Param_List_Minus_1)|| ", " || DatasetName;
      end;
      if last.Distribution then do;
         call symput(compress(Distribution || "_PARAM_LIST_MINUS_1"), Param_List_Minus_1);
      end;
   run;

   /***************************************************************/
   /*** setup Frequency Distributions Global Macro Vars          ***/
   /***************************************************************/
   %global MAX_FREQ_PARAMS;
   %let MAX_FREQ_PARAMS= 3;

   proc sort data=&infolib..sparmnames;
      by distribution;
   run;
   proc transpose data=&infolib..fparmnames out=_ifs_DistNames;
      var DatasetName;
      by distribution;
   run;
   data _NULL_; set _ifs_DistNames;
      array cols[*] col:;
      call symputX("MAX_FREQ_PARAMS", dim(cols));
      stop;
   run;
 
   %do i=1 %to &MAX_FREQ_PARAMS;
      %global fparam_&i._init;
      %global fparam_&i.;   
   %end;

   %global NQUANT;
   %global LNG_NQUANT;
   %global LNGROBUST_NQUANT;
   %global LNGROBUST_KRI_USED;
   %global NEEDED_INPUT;
   %global SCALE_TRANSFORM;
   %global DIST_NAME;
   %global PARAM_COUNT;
   %global ProbMeasure;
   %global COV_COUNT;
   %global NPARMS;
   %global SUMLOG; 
   %global CDF_TRANSFORM_FACTOR;
   %global BODY_DIST;
   %global TAIL_DIST;
   %global TAIL_PARAM_1;
   %global TAIL_PARAM_2;
   %global BODY_PARAM_COUNT;
   %global KRI_LOG_AVG;
   %global AVG_ADJ_GOF_T;     
   %global STATUS;      
   %global KRI_LOG;
   %global KRI_LIST;
   %global KRI_EFFECT;
   %global KRI_EFFECT_RETAIN;
   %global KRI_LOSS_EFFECT;
   %global KRI_RETAIN;
   %global KRI_AVG_EFFECT;
   %global Use_42_Scen_Calc;
   %global ORVP_STATUS;
   
   proc datasets library=work memtype=data nolist nodetails;
     delete _ifs_DistNames;
   quit;
   
%mend orva_init_globals;
