/*************************************************************************************
 * $Id: orva_loss_sim_stream.sas,v 1.1.4.17.4.7.2.4.8.1 2012/05/03 14:54:21 sasdje Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *    NAME: orva_loss_sim_stream.sas                                             
 * PURPOSE: sub-macro of %orva_loss_sim() called once for each stream.            
 * SUPPORT: jariou
 *
 * PARAMETERS:
 *    startrep    = 1st rep number.
 *    nreps       = Number of reps for stream.
 *    streamNum   = Current stream number.
 *    seed        = Initial seed for stream.
 *
 * NOTES: Please see the accompanying doc for %orva_loss_sim().
 *        The only output of this macro are the datasets:
 *           _osimNaked_&streamNum 
 *           _osimInsured_&streamNum
 *
 * SUBORDINATE MACROS:
 *    %ApplyInsurance(Level=);                                               
 *    %RunSimulation(lookupTable=, applyKris=);
 *    %ApplyPlainInsurance(Level=);
 *    %ApplyBaselInsurance(Level=);
 **************************************************************************************/
%macro orva_loss_sim_stream(
                            startRep  =, 
                            nReps     =,
                            streamNum =,
                            seed      =,
                            insuranceMethod=,
                            finald    = work
                            );

    data &finalD.._osimNaked_&streamNum (keep= _rep_ loss seed TotalLoss  TotalLoss2  TotalCount  TotalCount2 )
        &finalD.._osimInsured_&streamNum (Keep= _rep_ loss &InsuranceArray);
		  retain TotalLoss 0 TotalLoss2 0 TotalCount 0 TotalCount2 0;
    
       /* declare fake array to make sure insurance values are referenced
          otherwise, sas does not 'keep' them... */
       %if &InsuranceArray ne %str() %then
          array _dummy[*] &InsuranceArray;;
    
        /* Initialize the random stream */
        call streaminit(&seed);

        /* Always create a main lookup table array */
        array mainLookupTable(&lookupSize) _TEMPORARY_ (&LOOKUP_QUANTILES);


         %if "&coverageData1" ne "" and &CoverageCount^=0 %then %do;
                array Insurance_[&InsuranceContractCount] &InsuranceArray;
                array arInsurance_UID_1_[&CoverageCount] _TEMPORARY_ ( &Insurance_UID_1 );      
                array arInsurance_Index_1_[&CoverageCount] _TEMPORARY_ ( &Insurance_Index_1 );      
                array arDeductible_1_[&CoverageCount] _TEMPORARY_ ( &Deductible_1 );      
                array arLimit_1_[&CoverageCount] _TEMPORARY_ (&The_Limit_1 );                      
                array arEff_Agg_Limit_1_[&CoverageCount] _TEMPORARY_ (&Eff_Agg_Limit_1 );     
                array arPayout_Haircut_1_[&CoverageCount] _TEMPORARY_ (&Payout_Haircut_1 );    
                array arPolicyExpiryDate_1_[&CoverageCount] _TEMPORARY_ (&PolicyExpiryDate_1 );    
                array arInsurance_UID_2_[&CoverageCount] _TEMPORARY_ ( &Insurance_UID_2 );      
                array arInsurance_Index_2_[&CoverageCount] _TEMPORARY_ ( &Insurance_Index_2 );      
                array arDeductible_2_[&CoverageCount] _TEMPORARY_ (&Deductible_2 );            
                array arLimit_2_[&CoverageCount] _TEMPORARY_ (&The_Limit_2 );                      
                array arEff_Agg_Limit_2_[&CoverageCount] _TEMPORARY_ (&Eff_Agg_Limit_2 );              
                array arPayout_Haircut_2_[&CoverageCount] _TEMPORARY_ (&Payout_Haircut_2 );    
                array arPolicyExpiryDate_2_[&CoverageCount] _TEMPORARY_ (&PolicyExpiryDate_2 );    
        %end;


        /*  Later, I will set loss to sumNaked, then output to outsimNaked, 
            then set loss to sumInsured and output to outsimInsured.    */


            /* TODO: */ 
            /* Could replace the set _loss_ point=??? */
            /* by an array written to a file before   */
            /* slope = (lookupTable[&LookupSize] - lookupTable[&LookupSize-1]); */
 
        /* Loop through replications */

        do curr_rep = &startRep to &startRep + &nreps -1 ;
            set _loss_ point=curr_rep;

            %if "&coverageData1" ne "" and &CoverageCount^=0 %then %do;
                do ii =1 to &InsuranceContractCount;
                    Insurance_[ii]=0;
                end;
            %end;    
            /*- simulate severities -*/
            sumNaked =0;
            sumInsured=0;
            sumInsurance=0;

				TotalCount = TotalCount + nloss;
				TotalCount2 = TotalCount2 + nloss * nloss;

            /* Loop through individual losses */
            do i = 1 to nloss; 
                %if "&AVERAGE_KRI_EFFECT" NE "1" %then %do;
                    %RunSimulation( lookupTable = mainLookupTable, applyKris = Yes, insMethod = &insuranceMethod);
                %end;
                %else %do;
                    %RunSimulation( lookupTable = mainLookupTable, applyKris = No, insMethod = &insuranceMethod);
                %end;
            end; /* End of Loop through individual losses */
                    
            _rep_  = curr_rep;
            loss   = sumNaked;                  
            output &finalD.._osimNaked_&streamNum;

            loss   = sumInsured;   
            output &finalD.._osimInsured_&streamNum;
    
        end; /* End of Loop through replications */
        stop;  /* for having used "point" */
   run;
                        
   %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
      %put &orvar_error %sysfunc(sasmsg(&orvads, lostream_cant_sim_agg_loss_error, noquote, &streamNum));
      data &finalD.._osimNaked_&streamNum; set _NULL_; run;
      data &finalD.._ostatNaked_&streamNum; set _NULL_; run;
   %end;
   
%mend orva_loss_sim_stream;


%macro RunSimulation(lookupTable=, applyKris=, insMethod = );
                    uni       = rand("UNIFORM"); 
                    LossIndex = (&LookupSize+1)*(1-uni**.4);
                    FL_Index  = Floor(LossIndex);

                    if FL_Index EQ 0 then do;
                        iLoss = (1-(1-LossIndex)**2.5)*&lookupTable[1];
                    end;
                    else if FL_Index GE &LookupSize then do;
                        iLoss = &lookupTable[&LookupSize] + 
											(LossIndex-&LookupSize)*(&lookupTable[&LookupSize]-&lookupTable[&LookupSize-1]);      
                    end;
                    else do;
                        iLoss = &lookupTable[FL_Index] + 
											(LossIndex-FL_Index)*(&lookupTable[FL_Index+1]-&lookupTable[FL_Index]);                        
                    end;

                    /* TODO: */
                    /* Speed up can be accomplished here by putting all of this in an array */
                    /* Scale appropriately if KRI's are used */
                    %if "&applyKris" EQ "Yes" %then %do;
	                    uChooseKriScale = 1 + int(&KRI_CHOICES * rand("UNIFORM"));
	                    set _tmp_kri_ point = uChooseKriScale;
	                    iLoss = iLoss * KRI_SCALE_EFFECT/&AVERAGE_KRI_EFFECT;

                    %end;

						  TotalLoss  = TotalLoss + iLoss;
						  TotalLoss2 = TotalLoss2 + iLoss * iLoss;

		              if iLoss > &MaxLoss then do;
                       iLoss = &MaxLoss;
		              end;

                    sumNaked = sumNaked + iloss;

                    /*- apply insurance contract -*/
                    %if "&coverageData1" ne "" and &CoverageCount^=0 %then %do;  
								%orva_insurance_apply_&insMethod.();
                    %end; 
                    sumInsured = sumInsured + iloss;                
%mend;/* end of %RunSimulation */
