/*****************************************************************************************
 * $Id: orvl_int_model_scenario_32.sas,v 1.1.2.13.2.8.2.1.8.1 2012/05/10 17:40:53 fengye Exp $
 *  Copyright (C), 2006
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orvl_int_model_scenario_32                                             
 *  PURPOSE: Performs loading of the model scenario datasets.
 *  SUPPORT: mimuha
 *
 *  PARAMETERS:
 *    INLIB           = Libref for the input datasets
 *    OUTLIB          = Libref for the output datasets
 *    DATA_VERSION_ID = Version uid
 *    SEQ_LIB         = The libref for the sequences dataset
 *****************************************************************************************/
%macro orvl_int_model_scenario_32(INLIB, OUTLIB, DATA_VERSION_ID, SEQ_LIB);

   data _null_;
      call symput('TYP_NM', sasmsg("&orvlds", "ims_type_model_scen", "noquote"));
   run;
   data _null_;
      call symput('HEADER_SC', sasmsg("&orvlds", "ims_ref_missing_scenario_title", "noquote", "&TYP_NM"));
      call symput('HEADER_BL', sasmsg("&orvlds", "ims_ref_missing_int_bl_title", "noquote", "&TYP_NM"));
      call symput('HEADER_RC', sasmsg("&orvlds", "ims_ref_missing_int_rc_title", "noquote", "&TYP_NM"));
      call symput('HEADER_BLS', sasmsg("&orvlds", "ims_ref_missing_std_bl_title", "noquote", "&TYP_NM"));
      call symput('HEADER_RCS', sasmsg("&orvlds", "ims_ref_missing_std_rc_title", "noquote", "&TYP_NM"));
      call symput('HEADER_PR', sasmsg("&orvlds", "ims_ref_missing_process_title", "noquote", "&TYP_NM"));
      call symput('HEADER_LE', sasmsg("&orvlds", "ims_ref_missing_legal_entity_title", "noquote", "&TYP_NM"));
      call symput('HEADER_C1', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "1"));
      call symput('HEADER_C2', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "2"));
      call symput('HEADER_C3', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "3"));
      call symput('HEADER_C4', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "4"));
      call symput('HEADER_C5', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "5"));
      call symput('HEADER_C6', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "6"));
      call symput('HEADER_C7', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "7"));
      call symput('HEADER_C8', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "8"));
      call symput('HEADER_C9', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "9"));
      call symput('HEADER_C10', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "10"));
   run;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..model_scenario, "&TYP_NM", false);
   %if &ds_nobs eq 0 %then %do;
      %orvl_cmn_status_success;
      %return;
   %end;

   %orvl_log_output_wrap("STATUS - %sysfunc(sasmsg(&orvlds, ims_testing_model_scens_status, noquote))");

   %orvl_int_model_scenario_pcheck(ScenarioDataIn=&INLIB..model_scenario, ScenarioDataOut=WORK.model_scenario_32_ck);

   %orvl_log_output_wrap("STATUS PROGRESS2 TOTAL=&ds_nobs FILE='&TYP_NM'");

   /*
    * ***************************
    * Process the Model Scenarios. 
    * ***************************
    */
   /* Get the starting uids for items to insert. */
   /* SCENARIO_ID needs twice as many, to handle the column def'n details entry for the event. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Scenario_Details_SEQ, dbload, INC=&ds_nobs);
   proc sql noprint;
      select VALUE Into :SCENARIO_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;
   %orvdb_seq_incrementer(&SEQ_LIB, Folder_SEQ, dbload, INC=&ds_nobs);
   proc sql noprint;
      select VALUE Into :SCENARIO_EVENT_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   proc summary data=&OUTLIB..MODEL_SCENARIO_SEV_PARMS MAX;
      ID SEV_PARM_UID;
      OUTPUT OUT=WORK.TMP_ID_DS;
   run;
   %orvl_ds_exists(WORK.TMP_ID_DS, "ID_FOR_SEV_PARMS", false);
   %if &ds_nobs ne 0 %then %do;
      proc sql noprint;
         select SEV_PARM_UID Into :PARM_ID From WORK.TMP_ID_DS;
      quit;
   %end;
   %else %do;
      data _null_;
         call symput('PARM_ID', '0');
      run;
   %end;

   %orvl_cmn_status_ds_prep(WORK.LOAD_STATUS_DETAILS);

   data WORK.tmp_ev (KEEP= SCENARIO_EVENT_UID EVENT_NAME SCENARIO_UID);
      set &OUTLIB..SCENARIO_EVENT;
   run;
   data WORK.tmp_eff (KEEP= SCENARIO_EFFECT_UID EFFECT_NAME SCENARIO_EVENT_UID);
      set &OUTLIB..SCENARIO_EFFECT;
   run;

   /* Create a temporary dataset with the new ids. */
   data WORK.TMP_SCENARIO_DETAILS_M (KEEP= SCENARIO_DETAILS_UID
                                           FEED_REF_ID 
                                           PARENT_UID
                                           TYPE_CD
                                           INTERNAL_BL_UID
                                           INTERNAL_RC_UID
                                           BASEL_BL_UID
                                           BASEL_RC_UID
                                           PROCESS_UID
                                           LEGAL_ENTITY_UID
                                           CLASS_1
                                           CLASS_2
                                           CLASS_3
                                           CLASS_4
                                           CLASS_5
                                           CLASS_6
                                           CLASS_7
                                           CLASS_8
                                           CLASS_9
                                           CLASS_10
                                           PROBABILITY
                                           DATETIME_LOADED
                                           VERSION_UID)
        &OUTLIB..SCENARIO_EVENT
        &OUTLIB..SCENARIO_EFFECT
        &OUTLIB..MODEL_SCENARIO_SEV_PARMS
        WORK.LOAD_STATUS_DETAILS
        %orvl_log_ds_init();
      length ORVAR_REASON_OBS_PREFIX $40.;
      length SCENARIO_NAME EVENT_NAME EFFECT_NAME $100. SCENARIO_DESC $256. VALID_DISTS $1024. S_DIST $30. F_DIST $30.;
      format SCENARIO_TYPE $1. BL_AND_RC_TYPE $1.; 
      set WORK.model_scenario_32_ck (rename=(Model_Scenario_Entry_ID=in_id
                                             Scenario_ID=in_scenario_ref_id
                                             Internal_BL_ID=in_bl_ref_id
                                             Internal_RC_ID=in_rc_ref_id
                                             /* Process_ID=in_process_ref_id */
                                             /* Legal_Entity_ID=in_legal_entity_ref_id */
                                             Severity_Dist=in_sev_dist_name
                                             Frequency_Dist=in_freq_dist_name
                                             Fparm1=in_fparm0
                                             Fparm2=in_fparm1
                                             VALID=in_is_valid
                                             REASON=in_reason));

      retain scenario_id_val scenario_event_id_val sev_parm_id_val;

      /* Get this from the input variable before anything toasts it! */
      EFFECT_NAME = SCENARIO_NAME;
      
      if _N_ = 1 then do;
         scenario_id_val = symgetn('SCENARIO_ID');
         if (scenario_id_val eq .) then do;
            scenario_id_val = 1;
         end;
         scenario_event_id_val = symgetn('SCENARIO_EVENT_ID');
         if (scenario_event_id_val eq .) then do;
            scenario_event_id_val = 1;
         end;
         sev_parm_id_val = symgetn('PARM_ID') + 1;
         if (sev_parm_id_val eq .) then do;
            sev_parm_id_val = 1;
         end;
         declare hash scenarioHash(dataset: "&OUTLIB..SCENARIO", hashexp: 10);
         rc = scenarioHash.defineKey('FEED_REF_ID');
         rc = scenarioHash.defineData('SCENARIO_UID', 'SCENARIO_TYPE', 'SCENARIO_NAME', 'SCENARIO_DESC', 'BL_AND_RC_TYPE');
         rc = scenarioHash.defineDone();
         declare hash eventHash(dataset: "WORK.tmp_ev", hashexp: 8);
         rc = eventHash.defineKey('EVENT_NAME', 'SCENARIO_UID');
         rc = eventHash.defineData('SCENARIO_EVENT_UID');
         rc = eventHash.defineDone();
         declare hash effectHash(dataset: "WORK.tmp_eff", hashexp: 8);
         rc = effectHash.defineKey('EFFECT_NAME', 'SCENARIO_EVENT_UID');
         rc = effectHash.defineData('SCENARIO_EFFECT_UID');
         rc = effectHash.defineDone();
         declare hash blHash(dataset: "&OUTLIB..INTERNAL_BUSINESSLINE", hashexp: 10);
         rc = blHash.defineKey('FEED_REF_ID');
         rc = blHash.defineData('INTERNAL_BL_UID');
         rc = blHash.defineDone();
         declare hash rcHash(dataset: "&OUTLIB..INTERNAL_RISK_CATEGORY", hashexp: 10);
         rc = rcHash.defineKey('FEED_REF_ID');
         rc = rcHash.defineData('INTERNAL_RC_UID');
         rc = rcHash.defineDone();
         declare hash blBaselHash(dataset: "&OUTLIB..BASEL_BUSINESSLINE_I", hashexp: 10);
         rc = blBaselHash.defineKey('FEED_REF_ID');
         rc = blBaselHash.defineData('BASEL_BL_UID');
         rc = blBaselHash.defineDone();
         declare hash rcBaselHash(dataset: "&OUTLIB..BASEL_RISK_CATEGORY_I", hashexp: 10);
         rc = rcBaselHash.defineKey('FEED_REF_ID');
         rc = rcBaselHash.defineData('BASEL_ER_UID');
         rc = rcBaselHash.defineDone();
         declare hash processHash(dataset: "&OUTLIB..PROCESS", hashexp: 10);
         rc = processHash.defineKey('FEED_REF_ID');
         rc = processHash.defineData('PROCESS_UID');
         rc = processHash.defineDone();
         declare hash legalEntityHash(dataset: "&OUTLIB..LEGAL_ENTITY", hashexp: 10);
         rc = legalEntityHash.defineKey('FEED_REF_ID');
         rc = legalEntityHash.defineData('LEGAL_ENTITY_UID');
         rc = legalEntityHash.defineDone();
         declare hash distSev(dataset: "WORK._SDISTS", hashexp: 6);
         rc = distSev.defineKey('S_DIST');
         rc = distSev.defineData('S_DIST', 'S_NUM');
         rc = distSev.defineDone();
         declare hiter iter_s('distSev');
         declare hash distFreq(dataset: "WORK._FDISTS", hashexp: 6);
         rc = distFreq.defineKey('F_DIST');
         rc = distFreq.defineData('F_DIST', 'F_NUM');
         rc = distFreq.defineDone();
         %do i = 1 %to 10;
             declare hash class&i.Hash(dataset: "&OUTLIB..CLASS_&i.", hashexp: 10);
             rc = class&i.Hash.defineKey('FEED_REF_ID');
             rc = class&i.Hash.defineData("CLASS_&i._UID");
             rc = class&i.Hash.defineDone();
             call missing(CLASS_&i._UID);
         %end;
         declare hiter iter_f('distFreq');
         %orvl_cmn_obs_exists_hash(OBS_EXISTS_EXISTING_COUNT=0, 
                                   HASH_KEY='FEED_REF_ID',
                                   HASH_DATA='scenario_id_val');
         call missing(FEED_REF_ID, SCENARIO_UID, SCENARIO_TYPE, SCENARIO_NAME, SCENARIO_DESC, BL_AND_RC_TYPE);
         call missing(EVENT_NAME, SCENARIO_EVENT_UID);
         call missing(INTERNAL_BL_UID, INTERNAL_RC_UID, BASEL_BL_UID);
         call missing(BASEL_ER_UID, PROCESS_UID, LEGAL_ENTITY_UID, S_DIST, S_NUM, F_DIST, F_NUM);
      end;
      else do;
         scenario_id_val = scenario_id_val + 1;
         scenario_event_id_val = scenario_event_id_val + 1;
         sev_parm_id_val = sev_parm_id_val + 1;
      end;

      %orvl_log_ds_modify();
      %orvl_cmn_status_ds_modify(WORK.LOAD_STATUS_DETAILS);

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      onErrorFail = 1;

      if 0 then modify &OUTLIB..SCENARIO_EVENT;
      if 0 then modify &OUTLIB..SCENARIO_EFFECT;
      if 0 then modify &OUTLIB..MODEL_SCENARIO_SEV_PARMS;

      FEED_REF_ID = in_scenario_ref_id;
      rc = scenarioHash.find();
      if (rc ne 0) then do;
         %orvl_find_reference(rc, "&HEADER_SC", WORK.LOAD_STATUS_DETAILS);
         goto SKIPPER;
      end;
      else if (strip(SCENARIO_TYPE) ne "M") then do;
         %orvl_log_output("%sysfunc(sasmsg(&orvlds, ims_change_scen_error, noquote))");
         WHO = "&TYP_NM";
         STATUS = 5;
         %orvl_cmn_set_reason_prefix(_N_);
         REASON = sasmsg("&orvlds", "ims_scen_type_error", "noquote", 
                         strip(ORVAR_REASON_OBS_PREFIX), strip(SCENARIO_TYPE));
         call symputx('LOAD_STATUS', STATUS);
         call symput('LOAD_REASON', REASON);
         output WORK.LOAD_STATUS_DETAILS;
         REASON = "STATUS WARNING: " || strip(REASON);
         %orvl_log_output(strip(REASON));
         goto SKIPPER;
      end;

      doOutputEvent = 0;
      EVENT_NAME = STRIP(SCENARIO_NAME);
      rc = eventHash.find();
      if (rc ne 0) then do;
         SCENARIO_EVENT_UID = scenario_event_id_val;
         eventHash.add();
         doOutputEvent = 1;
      end;

      /* Just make sure the variables get created if they are not in the input datasets. */
      %do i = 1 %to 10;   
         CLASS_&i. = CLASS_&i.;
      %end;
      FREQUENCY_MIN = FREQUENCY_MIN;
      FREQUENCY_MAX = FREQUENCY_MAX;

      /* Assign whatever variables are available. */
      EVENT_DESC = SCENARIO_DESC;
      EVENT_TYPE = 0;
      FREQUENCY_DIST = in_freq_dist_name;
      FREQUENCY_PARM0 = in_fparm0;
      FREQUENCY_PARM1 = in_fparm1;
      FREQUENCY_PARM2 = .;

      SCENARIO_EFFECT_UID = scenario_event_id_val;
      if (strip(EFFECT_NAME) eq '') then 
         EFFECT_NAME = 'Effect - ' || put(SCENARIO_EFFECT_UID, BEST12.-L);
      EFFECT_DESC = EFFECT_NAME;
      EFFECT_ORDER = 0;
      EFFECT_PROB = 1;
      PROBABILITY = 1;
      BUCKET_UID = .;
      SEVERITY_DIST = in_sev_dist_name;
      SEVERITY_STATUS_REASON = '';
      SIMPLE_FREQUENCY_DIST = in_freq_dist_name;
      SIMPLE_FREQUENCY_PARM0 = in_fparm0;
      SIMPLE_FREQUENCY_PARM1 = in_fparm1;
      SIMPLE_FREQUENCY_PARM2 = .;

      SCENARIO_DETAILS_UID = scenario_id_val;
      PARENT_UID = SCENARIO_EFFECT_UID;
      TYPE_CD = 'E';
      VERSION_UID = &DATA_VERSION_ID;

      /* Create a flag that will be used to determine if every item was found. */
      doOutput = 1;
            
      /* Find the Business Line and Risk Category. */
      if (strip(BL_AND_RC_TYPE) eq "I") then do;
         FEED_REF_ID = in_bl_ref_id;
         rc = blHash.find();
         if (FEED_REF_ID ne .) then do;
            %orvl_find_reference(rc, "&HEADER_BL", WORK.LOAD_STATUS_DETAILS);
            INTERNAL_BL_UID = INTERNAL_BL_UID;
         end;
   
         FEED_REF_ID = in_rc_ref_id;
         rc = rcHash.find();
         if (FEED_REF_ID ne .) then do;
            %orvl_find_reference(rc, "&HEADER_RC", WORK.LOAD_STATUS_DETAILS);
            INTERNAL_RC_UID = INTERNAL_RC_UID;
         end;
      end;
      else do;
         FEED_REF_ID = in_bl_ref_id;
         rc = blBaselHash.find();
         if (FEED_REF_ID ne .) then do;
            %orvl_find_reference(rc, "&HEADER_BLS", WORK.LOAD_STATUS_DETAILS);
            BASEL_BL_UID = BASEL_BL_UID;
         end;
   
         FEED_REF_ID = in_rc_ref_id;
         rc = rcBaselHash.find();
         if (FEED_REF_ID ne .) then do;
            %orvl_find_reference(rc, "&HEADER_RCS", WORK.LOAD_STATUS_DETAILS);
            BASEL_RC_UID = BASEL_ER_UID;
         end;
      end;

      /* Find the Process. */
      Process_ID = Process_ID;
      FEED_REF_ID = Process_ID;
      rc = processHash.find();
      if (FEED_REF_ID ne .) then do;
         %orvl_find_reference(rc, "&HEADER_PR", WORK.LOAD_STATUS_DETAILS);
         PROCESS_UID = PROCESS_UID;
      end;

      /* Find the LegalEntity. */
      FEED_REF_ID = Legal_Entity_Id;
      rc = legalEntityHash.find();
      if (FEED_REF_ID ne .) then do;
         %orvl_find_reference(rc, "&HEADER_LE", WORK.LOAD_STATUS_DETAILS);
         LEGAL_ENTITY_UID = LEGAL_ENTITY_UID;
      end;

      %do i = 1 %to 10;   
         if (CLASS_&i. ne .) then do;
            /* Find the CLASS_# Refs. */
            FEED_REF_ID = CLASS_&i.;
            rc = class&i.Hash.find();
            %orvl_find_reference(rc, "&&HEADER_C&i", WORK.LOAD_STATUS_DETAILS);
            CLASS_&i. = CLASS_&i._UID;
         end;
      %end;

      /* Make sure the Severity and Frequency Distributions are valid. */
      if (strip(FREQUENCY_DIST) ne "") then do;
         F_DIST = FREQUENCY_DIST;
         rc = distFreq.find();
         if (rc ^= 0) then do;
            VALID_DISTS = " ";
            iter_rc = iter_f.first();
            do while (iter_rc = 0);
               VALID_DISTS = strip(VALID_DISTS) || " " || F_DIST;
               iter_rc = iter_f.next();
            end;
            VALID_DISTS = sasmsg("&orvlds", "ims_invalid_dist_for_freq", "noquote", 
                                 strip(FREQUENCY_DIST), strip(VALID_DISTS));
            %orvl_cmn_record_error(43, _N_, strip(VALID_DISTS), "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
         end;
      end;         

      if (strip(SEVERITY_DIST) ne "") then do;
         S_DIST = SEVERITY_DIST;
         rc = distSev.find();
         if (rc ^= 0) then do;
            VALID_DISTS = " ";
            iter_rc = iter_s.first();
            do while (iter_rc = 0);
               VALID_DISTS = strip(VALID_DISTS) || " " || S_DIST;
               iter_rc = iter_s.next();
            end;
            VALID_DISTS = sasmsg("&orvlds", "ims_invalid_dist_for_sev", "noquote", 
                                 strip(SEVERITY_DIST), strip(VALID_DISTS));
            %orvl_cmn_record_error(43, _N_, strip(VALID_DISTS), "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
         end;
      end;         

      /* Are the parameters valid? */
      if (in_is_valid ne 1) then do;
         %orvl_cmn_record_error(40, _N_, in_reason, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
      end;

      FEED_REF_ID = in_id;
      %orvl_cmn_test_required_value(FEED_REF_ID, %str(Model_Scenario_Entry_ID), _N_, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);

      %orvl_cmn_obs_exists_test_msg(OBS_EXISTS_EXISTING_COUNT=0, SRC_NAME="&TYP_NM", DO_MSG_ONLY=false);

      /* Output the item and the parameters only if all of the referenced components 
       * were found, and the parameters are valid. */
      if doOutput eq 1 then do;
         DATETIME_LOADED = &ORVAR_DTTM_LOADED;
         output WORK.TMP_SCENARIO_DETAILS_M;
         if (doOutputEvent eq 1) then
            output &OUTLIB..SCENARIO_EVENT;

         rc = effectHash.find();
         SCENARIO_EFFECT_UID = scenario_event_id_val;
         if (rc eq 0) then do;
            EFFECT_NAME = SUBSTRN(STRIP(EFFECT_NAME), 1, 87) || " " || put(SCENARIO_EFFECT_UID, BEST12.-L);
         end;
         _ERROR_ = 0;
         effectHash.add();
         output &OUTLIB..SCENARIO_EFFECT;

         VERSION_UID = &DATA_VERSION_ID;
         Array sev_parm_ref[100] Sparm1 - Sparm100;
         i = 1;
         do while ((i <= 100) and (sev_parm_ref[i] ne .));
            SEV_PARM_UID = sev_parm_id_val;
            PARM_NUM = i;
            PARM_VALUE = sev_parm_ref[i];
            output &OUTLIB..MODEL_SCENARIO_SEV_PARMS;
            sev_parm_id_val = sev_parm_id_val + 1;
            i = i + 1;
         end;
      end;

SKIPPER:
	run;

   %orvl_ds_exists(&OUTLIB..SCENARIO, "SCENARIO_COUNT", false);
   %orvl_int_scenario_col_def(WORK.TMP_SCENARIO_DETAILS_M, 
                              &OUTLIB..SCENARIO_EFFECT,
                              &OUTLIB..SCENARIO_EVENT,
                              &OUTLIB..SCENARIO,
                              WORK.TMP_SCENARIO_DETAILS_DEF, 
                              &ds_nobs,
                              &SEQ_LIB);

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   /* Append the temp datasets to the original datasets. */
   %if (&LOAD_STATUS eq 1) %then %do;
      %orvl_modify_scenario_details("&TYP_NM [Column Defs]", &OUTLIB, WORK.TMP_SCENARIO_DETAILS_DEF);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   %end;

   %if (&LOAD_STATUS eq 1) %then %do;
      %orvl_modify_scenario_details("&TYP_NM [Details]", &OUTLIB, WORK.TMP_SCENARIO_DETAILS_M);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   %end;

%DONE:
   %orvl_log_output_wrap("STATUS PROGRESS2 DONE");

   /* Handle the final signaling. */
   %orvl_cmn_status_processing;

%mend orvl_int_model_scenario_32;
