/*****************************************************************************************
 *  Copyright (C), 2012
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orvl_modify_scenario_details                                             
 *  PURPOSE: Merge data from a temp dataset to a scenario details data set.
 *  SUPPORT: fengye
 *
 *  PARAMETERS:
 *    TYP_NM  = A title for display and error processing (the type of data being processed
 *    OUTLIB = Libname of the scenario details dataset being appended to
 *    DS_DATA = The dataset with the source data containg new data
 *    
 *****************************************************************************************/
%macro orvl_modify_scenario_details(TYP_NM, OUTLIB, DS_DATA);

   %orvl_log_output_wrap("STATUS - %sysfunc(sasmsg(&orvlds, cpa_appending_status, noquote, &TYP_NM))");
   %orvl_ds_exists(&DS_DATA, "New Data Size", false);
   
   %if &ds_nobs eq 0 %then %goto DONE; /* nothing to append or modify */
   
   %let numToModify = &ds_nobs;

   
   %let THE_NAME = foo;
   data _null_;
      call symput('THE_NAME', %UNQUOTE(&TYP_NM));
   run;
   %orvl_log_output_wrap("STATUS PROGRESS2 TOTAL=&numToModify FILE='&THE_NAME'");
   
   
   %orvl_ds_exists(&OUTLIB..SCENARIO_DETAILS, "&TYP_NM", false);
   %let existing_det_count = &ds_nobs;
   %if &existing_det_count eq 0 %then %do;
      %let if_modify_3 = if 0 then modify;
   %end;
   %else %do;
      %let if_modify_3 = modify key=FEED_ID_TYPE;
   %end;
   
   
   
   /* creating index FEED_ID_TYPE in order to perform modify */

   proc sql;
      drop index FEED_ID_TYPE from &OUTLIB..SCENARIO_DETAILS;
      create index FEED_ID_TYPE on &OUTLIB..SCENARIO_DETAILS(FEED_REF_ID, TYPE_CD);
      drop index FEED_ID_TYPE from &DS_DATA;
      create index FEED_ID_TYPE on &DS_DATA(FEED_REF_ID, TYPE_CD);
   quit;
 
 
   data &OUTLIB..SCENARIO_DETAILS WORK.LOAD_STATUS_DETAILS;
      set &DS_DATA (rename=(PARENT_UID=in_parent
                       INTERNAL_BL_UID=int_bl_id
                       INTERNAL_RC_UID=int_rc_id
                       BASEL_BL_UID=b_bl_id
                       BASEL_RC_UID=b_rc_id
                       PROCESS_UID=proc_id
                       LEGAL_ENTITY_UID=lg_id
                       CLASS_1=c1
                       CLASS_2=c2
                       CLASS_3=c3
                       
                       CLASS_4=c4
                       CLASS_5=c5
                       CLASS_6=c6
                       CLASS_7=c7
                       CLASS_8=c8
                       CLASS_9=c9
                       CLASS_10=c10
                       DATETIME_LOADED=date
                       VERSION_UID=vid));
     /* %orvl_log_ds_modify();  */
      &if_modify_3 &OUTLIB..SCENARIO_DETAILS;
      main_iorc = _IORC_;
      %orvl_cmn_status_ds_modify(WORK.LOAD_STATUS_DETAILS);
      
      PARENT_UID=in_parent;
      INTERNAL_BL_UID=int_bl_id;
      INTERNAL_RC_UID=int_rc_id;
      BASEL_BL_UID=b_bl_id;
      BASEL_RC_UID=b_rc_id;
      PROCESS_UID=proc_id;
      LEGAL_ENTITY_UID=lg_id;
      CLASS_1=c1;
      CLASS_2=c2;
      CLASS_3=c3;
      CLASS_4=c4;
      CLASS_5=c5;
      CLASS_6=c6;
      CLASS_7=c7;
      CLASS_8=c8;
      CLASS_9=c9;
      CLASS_10=c10;
      DATETIME_LOADED=date;
      VERSION_UID=vid;
      
      if ((&existing_det_count eq 0) or (main_iorc eq %sysrc(_DSENMR)) or (main_iorc eq %sysrc(_DSENOM))) then do;

	      output &OUTLIB..SCENARIO_DETAILS;
	      _ERROR_ = 0;
	   end;
	   else if ((main_iorc eq %sysrc(_SOK)) or (main_iorc eq %sysrc(_SENOCHN))) then do;

 	      replace &OUTLIB..SCENARIO_DETAILS;
 	      _ERROR_ = 0;
	   end;
	   %orvl_cmn_user_cancelled_ck_wrap();
	   %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   run;
      


%DONE:
   %orvl_log_output_wrap("STATUS PROGRESS2 DONE");

%mend orvl_modify_scenario_details;
